@@ -13,6 +13,7 @@ from dagster import check, seven
 from dagster.cli.workspace.cli_target import (
     get_repository_location_from_kwargs,
     get_repository_origin_from_kwargs,
+    get_working_directory_from_kwargs,
     python_origin_target_argument,
     repository_target_argument,
 )
@@ -55,13 +56,14 @@ from dagster.core.snap.execution_plan_snapshot import (
     ExecutionPlanSnapshotErrorData,
 )
 from dagster.core.storage.tags import check_tags
+from dagster.core.telemetry import telemetry_wrapper
 from dagster.core.types.loadable_target_origin import LoadableTargetOrigin
 from dagster.grpc import DagsterGrpcServer
 from dagster.grpc.impl import (
+    get_external_executable_params,
     get_external_execution_plan_snapshot,
     get_external_pipeline_subset_result,
     get_external_schedule_execution,
-    get_external_triggered_execution_params,
     get_partition_config,
     get_partition_names,
     get_partition_set_execution_param_data,
@@ -71,8 +73,8 @@ from dagster.grpc.types import (
     ExecuteRunArgs,
     ExecuteStepArgs,
     ExecutionPlanSnapshotArgs,
+    ExternalExecutableArgs,
     ExternalScheduleExecutionArgs,
-    ExternalTriggeredExecutionArgs,
     ListRepositoriesInput,
     ListRepositoriesResponse,
     LoadableRepositorySymbol,
@@ -88,12 +90,8 @@ from dagster.serdes import (
     serialize_dagster_namedtuple,
     whitelist_for_serdes,
 )
-from dagster.serdes.ipc import (
-    ipc_write_stream,
-    ipc_write_unary_response,
-    read_unary_input,
-    setup_interrupt_support,
-)
+from dagster.serdes.ipc import ipc_write_stream, ipc_write_unary_response, read_unary_input
+from dagster.utils import delay_interrupts, setup_windows_interrupt_support
 from dagster.utils.error import SerializableErrorInfo, serializable_error_info_from_exc_info
 from dagster.utils.hosted_user_process import (
     recon_pipeline_from_origin,
@@ -277,17 +275,17 @@ def schedule_execution_data_command(args):
 
 
 @unary_api_cli_command(
-    name="trigger_execution_params",
+    name="executable_params",
     help_str=(
         "[INTERNAL] Return the execution params for a triggered execution. This is an internal "
         "utility. Users should generally not invoke this command interactively."
     ),
-    input_cls=ExternalTriggeredExecutionArgs,
+    input_cls=ExternalExecutableArgs,
     output_cls=(ExternalExecutionParamsData, ExternalExecutionParamsErrorData),
 )
-def trigger_execution_params_command(args):
+def executable_params_command(args):
     recon_repo = recon_repository_from_origin(args.repository_origin)
-    return get_external_triggered_execution_params(recon_repo, args)
+    return get_external_executable_params(recon_repo, args)
 
 
 @whitelist_for_serdes
@@ -305,17 +303,20 @@ class ExecuteRunArgsLoadComplete(namedtuple("_ExecuteRunArgsLoadComplete", "")):
 @click.argument("input_file", type=click.Path())
 @click.argument("output_file", type=click.Path())
 def execute_run_command(input_file, output_file):
-    args = check.inst(read_unary_input(input_file), ExecuteRunArgs)
-    recon_pipeline = recon_pipeline_from_origin(args.pipeline_origin)
-    with DagsterInstance.from_ref(args.instance_ref) as instance:
-        with ipc_write_stream(output_file) as ipc_stream:
+    # Ensure that interrupts from the run launcher only happen inside user code or specially
+    # designated checkpoints
+    with delay_interrupts():
+        args = check.inst(read_unary_input(input_file), ExecuteRunArgs)
+        recon_pipeline = recon_pipeline_from_origin(args.pipeline_origin)
+        with DagsterInstance.from_ref(args.instance_ref) as instance:
+            with ipc_write_stream(output_file) as ipc_stream:
 
-            def send_to_stream(event):
-                ipc_stream.send(event)
+                def send_to_stream(event):
+                    ipc_stream.send(event)
 
-            return _execute_run_command_body(
-                recon_pipeline, args.pipeline_run_id, instance, send_to_stream
-            )
+                return _execute_run_command_body(
+                    recon_pipeline, args.pipeline_run_id, instance, send_to_stream
+                )
 
 
 @click.command(
@@ -363,11 +364,15 @@ def _execute_run_command_body(recon_pipeline, pipeline_run_id, instance, write_s
 
     # Perform setup so that termination of the execution will unwind and report to the
     # instance correctly
-    setup_interrupt_support()
+    setup_windows_interrupt_support()
 
     try:
         for event in execute_run_iterator(recon_pipeline, pipeline_run, instance):
             write_stream_fn(event)
+    except KeyboardInterrupt:
+        instance.report_engine_event(
+            message="Pipeline execution terminated by interrupt", pipeline_run=pipeline_run,
+        )
     except DagsterSubprocessError as err:
         if not all(
             [err_info.cls_name == "KeyboardInterrupt" for err_info in err.subprocess_error_infos]
@@ -464,14 +469,14 @@ def _schedule_tick_state(instance, stream, tick_data):
     "-p",
     type=click.INT,
     required=False,
-    help="Port over which to serve. You must pass one and only one of --port/-p or --socket/-f.",
+    help="Port over which to serve. You must pass one and only one of --port/-p or --socket/-s.",
 )
 @click.option(
     "--socket",
     "-s",
     type=click.Path(),
     required=False,
-    help="Serve over a UDS socket. You must pass one and only one of --port/-p or --socket/-f.",
+    help="Serve over a UDS socket. You must pass one and only one of --port/-p or --socket/-s.",
 )
 @click.option(
     "--host",
@@ -502,7 +507,16 @@ def _schedule_tick_state(instance, stream, tick_data):
     type=click.INT,
     required=False,
     default=30,
-    help="Timout after which to shutdown if --heartbeat is set and a heartbeat is not received",
+    help="Timeout after which to shutdown if --heartbeat is set and a heartbeat is not received",
+)
+@click.option(
+    "--lazy-load-user-code",
+    is_flag=True,
+    required=False,
+    default=False,
+    help="Wait until the first LoadRepositories call to actually load the repositories, instead of "
+    "waiting to load them when the server is launched. Useful for surfacing errors when the server "
+    "is managed directly from Dagit",
 )
 @python_origin_target_argument
 def grpc_command(
@@ -512,21 +526,22 @@ def grpc_command(
     max_workers=1,
     heartbeat=False,
     heartbeat_timeout=30,
+    lazy_load_user_code=False,
     **kwargs
 ):
     if seven.IS_WINDOWS and port is None:
         raise click.UsageError(
-            "You must pass a valid --port/-p on Windows: --socket/-f not supported."
+            "You must pass a valid --port/-p on Windows: --socket/-s not supported."
         )
     if not (port or socket and not (port and socket)):
-        raise click.UsageError("You must pass one and only one of --port/-p or --socket/-f.")
+        raise click.UsageError("You must pass one and only one of --port/-p or --socket/-s.")
 
     loadable_target_origin = None
     if any(kwargs[key] for key in ["attribute", "working_directory", "module_name", "python_file"]):
         loadable_target_origin = LoadableTargetOrigin(
             executable_path=sys.executable,
             attribute=kwargs["attribute"],
-            working_directory=kwargs["working_directory"],
+            working_directory=get_working_directory_from_kwargs(kwargs),
             module_name=kwargs["module_name"],
             python_file=kwargs["python_file"],
         )
@@ -539,6 +554,7 @@ def grpc_command(
         max_workers=max_workers,
         heartbeat=heartbeat,
         heartbeat_timeout=heartbeat_timeout,
+        lazy_load_user_code=lazy_load_user_code,
     )
 
     server.serve()
@@ -602,6 +618,7 @@ def launch_scheduled_execution(output_file, schedule_name, **kwargs):
                     )
 
 
+@telemetry_wrapper
 def _launch_scheduled_execution(
     instance, repo_location, external_repo, external_schedule, tick, stream
 ):
@@ -727,7 +744,7 @@ def create_api_cli_group():
     group.add_command(partition_names_command)
     group.add_command(partition_set_execution_param_command)
     group.add_command(schedule_execution_data_command)
-    group.add_command(trigger_execution_params_command)
+    group.add_command(executable_params_command)
     group.add_command(launch_scheduled_execution)
     group.add_command(grpc_command)
     return group
