@@ -1,29 +1,24 @@
-import os
 from collections import namedtuple
-from glob import glob
 
 import pkg_resources
 import six
 import yaml
 
 from dagster import check
-from dagster.core.errors import DagsterInvalidDefinitionError, DagsterInvariantViolationError
+from dagster.core.definitions.utils import config_from_files, config_from_yaml_strings
+from dagster.core.errors import DagsterInvariantViolationError
 from dagster.seven import FileNotFoundError, ModuleNotFoundError  # pylint:disable=redefined-builtin
-from dagster.utils import merge_dicts
-from dagster.utils.backcompat import (
-    canonicalize_backcompat_args,
-    canonicalize_run_config,
-    rename_warning,
-)
-from dagster.utils.yaml_utils import merge_yaml_strings, merge_yamls
+from dagster.utils.backcompat import canonicalize_backcompat_args
+from dagster.utils.merger import deep_merge_dicts
 
 from .mode import DEFAULT_MODE_NAME
+from .utils import check_for_invalid_name_and_warn
 
 
 class PresetDefinition(
-    namedtuple('_PresetDefinition', 'name run_config solid_selection mode tags')
+    namedtuple("_PresetDefinition", "name run_config solid_selection mode tags")
 ):
-    '''Defines a preset configuration in which a pipeline can execute.
+    """Defines a preset configuration in which a pipeline can execute.
 
     Presets can be used in Dagit to load predefined configurations into the tool.
 
@@ -48,35 +43,28 @@ class PresetDefinition(
             solid names) to execute with the preset. e.g. ``['*some_solid+', 'other_solid']``
         mode (Optional[str]): The mode to apply when executing this preset. (default: 'default')
         tags (Optional[Dict[str, Any]]): The tags to apply when executing this preset.
-    '''
+    """
 
     def __new__(
-        cls,
-        name,
-        run_config=None,
-        solid_selection=None,
-        mode=None,
-        environment_dict=None,
-        tags=None,
+        cls, name, run_config=None, solid_selection=None, mode=None, tags=None,
     ):
-        run_config = canonicalize_run_config(run_config, environment_dict)
 
         return super(PresetDefinition, cls).__new__(
             cls,
-            name=check.str_param(name, 'name'),
+            name=check_for_invalid_name_and_warn(name),
             run_config=run_config,
             solid_selection=check.opt_nullable_list_param(
-                solid_selection, 'solid_selection', of_type=str
+                solid_selection, "solid_selection", of_type=str
             ),
-            mode=check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME),
-            tags=check.opt_dict_param(tags, 'tags', key_type=str),
+            mode=check.opt_str_param(mode, "mode", DEFAULT_MODE_NAME),
+            tags=check.opt_dict_param(tags, "tags", key_type=str),
         )
 
     @staticmethod
     def from_files(
         name, environment_files=None, config_files=None, solid_selection=None, mode=None, tags=None
     ):
-        '''Static constructor for presets from YAML files.
+        """Static constructor for presets from YAML files.
 
         Args:
             name (str): The name of this preset. Must be unique in the presets defined on a given
@@ -95,46 +83,24 @@ class PresetDefinition(
         Raises:
             DagsterInvariantViolationError: When one of the YAML files is invalid and has a parse
                 error.
-        '''
-        check.str_param(name, 'name')
+        """
+        check.str_param(name, "name")
         config_files = canonicalize_backcompat_args(
-            config_files, 'config_files', environment_files, 'environment_files', '0.9.0'
+            config_files, "config_files", environment_files, "environment_files", "0.9.0"
         )
-        config_files = check.opt_list_param(config_files, 'config_files')
+        config_files = check.opt_list_param(config_files, "config_files")
         solid_selection = check.opt_nullable_list_param(
-            solid_selection, 'solid_selection', of_type=str
+            solid_selection, "solid_selection", of_type=str
         )
-        mode = check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME)
+        mode = check.opt_str_param(mode, "mode", DEFAULT_MODE_NAME)
 
-        filenames = []
-        for file_glob in environment_files or []:
-            globbed_files = glob(file_glob)
-            if not globbed_files:
-                raise DagsterInvalidDefinitionError(
-                    'File or glob pattern "{file_glob}" for "environment_files" in preset '
-                    '"{name}" produced no results.'.format(name=name, file_glob=file_glob)
-                )
-
-            filenames += [os.path.realpath(globbed_file) for globbed_file in globbed_files]
-
-        try:
-            merged = merge_yamls(filenames)
-        except yaml.YAMLError as err:
-            six.raise_from(
-                DagsterInvariantViolationError(
-                    'Encountered error attempting to parse yaml. Parsing files {file_set} '
-                    'loaded by file/patterns {files} on preset "{name}".'.format(
-                        file_set=filenames, files=environment_files, name=name
-                    )
-                ),
-                err,
-            )
+        merged = config_from_files(config_files)
 
         return PresetDefinition(name, merged, solid_selection, mode, tags)
 
     @staticmethod
     def from_yaml_strings(name, yaml_strings=None, solid_selection=None, mode=None, tags=None):
-        '''Static constructor for presets from YAML strings.
+        """Static constructor for presets from YAML strings.
 
         Args:
             name (str): The name of this preset. Must be unique in the presets defined on a given
@@ -153,24 +119,15 @@ class PresetDefinition(
         Raises:
             DagsterInvariantViolationError: When one of the YAML documents is invalid and has a
                 parse error.
-        '''
-        check.str_param(name, 'name')
-        yaml_strings = check.opt_list_param(yaml_strings, 'yaml_strings', of_type=str)
+        """
+        check.str_param(name, "name")
+        yaml_strings = check.opt_list_param(yaml_strings, "yaml_strings", of_type=str)
         solid_selection = check.opt_nullable_list_param(
-            solid_selection, 'solid_selection', of_type=str
+            solid_selection, "solid_selection", of_type=str
         )
-        mode = check.opt_str_param(mode, 'mode', DEFAULT_MODE_NAME)
+        mode = check.opt_str_param(mode, "mode", DEFAULT_MODE_NAME)
 
-        try:
-            merged = merge_yaml_strings(yaml_strings)
-        except yaml.YAMLError as err:
-            six.raise_from(
-                DagsterInvariantViolationError(
-                    'Encountered error attempting to parse yaml. Parsing YAMLs {yaml_strings} '
-                    'on preset "{name}".'.format(yaml_strings=yaml_strings, name=name)
-                ),
-                err,
-            )
+        merged = config_from_yaml_strings(yaml_strings)
 
         return PresetDefinition(name, merged, solid_selection, mode, tags)
 
@@ -178,7 +135,7 @@ class PresetDefinition(
     def from_pkg_resources(
         name, pkg_resource_defs=None, solid_selection=None, mode=None, tags=None
     ):
-        '''Load a preset from a package resource, using :py:func:`pkg_resources.resource_string`.
+        """Load a preset from a package resource, using :py:func:`pkg_resources.resource_string`.
 
         Example:
 
@@ -212,9 +169,9 @@ class PresetDefinition(
         Raises:
             DagsterInvariantViolationError: When one of the YAML documents is invalid and has a
                 parse error.
-        '''
+        """
         pkg_resource_defs = check.opt_list_param(
-            pkg_resource_defs, 'pkg_resource_defs', of_type=tuple
+            pkg_resource_defs, "pkg_resource_defs", of_type=tuple
         )
 
         try:
@@ -225,8 +182,8 @@ class PresetDefinition(
         except (ModuleNotFoundError, FileNotFoundError, UnicodeDecodeError) as err:
             six.raise_from(
                 DagsterInvariantViolationError(
-                    'Encountered error attempting to parse yaml. Loading YAMLs from '
-                    'package resources {pkg_resource_defs} '
+                    "Encountered error attempting to parse yaml. Loading YAMLs from "
+                    "package resources {pkg_resource_defs} "
                     'on preset "{name}".'.format(pkg_resource_defs=pkg_resource_defs, name=name)
                 ),
                 err,
@@ -235,17 +192,17 @@ class PresetDefinition(
         return PresetDefinition.from_yaml_strings(name, yaml_strings, solid_selection, mode, tags)
 
     def get_environment_yaml(self):
-        '''Get the environment dict set on a preset as YAML.
+        """Get the environment dict set on a preset as YAML.
 
         Returns:
             str: The environment dict as YAML.
-        '''
+        """
         return yaml.dump(self.run_config or {}, default_flow_style=False)
 
     def with_additional_config(self, run_config):
-        '''Return a new PresetDefinition with additional config merged in to the existing config.'''
+        """Return a new PresetDefinition with additional config merged in to the existing config."""
 
-        check.opt_nullable_dict_param(run_config, 'run_config')
+        check.opt_nullable_dict_param(run_config, "run_config")
         if run_config is None:
             return self
         else:
@@ -254,11 +211,5 @@ class PresetDefinition(
                 solid_selection=self.solid_selection,
                 mode=self.mode,
                 tags=self.tags,
-                run_config=merge_dicts(self.run_config, run_config),
+                run_config=deep_merge_dicts(self.run_config, run_config),
             )
-
-    @property
-    def environment_dict(self):
-        # Backcompat
-        rename_warning('run_config', 'environment_dict', '0.9.0')
-        return self.run_config
